@@ -124,7 +124,7 @@ class RandomVelocities(SnapshotModifier):
     are all in the same unit system. In particular, ``1.0 / beta * masses``
     must be in units of ``velocity**2``.
 
-    For the OpenMMEngine, for example (after ``from simtk import unit as
+    For the OpenMMEngine, for example (after ``from openmm import unit as
     u``), the ``beta`` parameter for 300 K would be created with
 
     .. code-block:: python
@@ -133,7 +133,7 @@ class RandomVelocities(SnapshotModifier):
 
     Parameters
     ----------
-    beta : float or simtk.unit.Quantity
+    beta : float or openmm.unit.Quantity
         inverse temperature (including kB) for the distribution
     engine : :class:`.DynamicsEngine` or None
         engine to be used for constraints; if None, use the snapshot's
@@ -247,10 +247,11 @@ class GeneralizedDirectionModifier(SnapshotModifier):
     SingleAtomVelocityDirectionModifier
     """
     def __init__(self, delta_v, subset_mask=None,
-                 remove_linear_momentum=True):
+                 remove_linear_momentum=True, engine=None):
         super(GeneralizedDirectionModifier, self).__init__(subset_mask)
         self.delta_v = delta_v
         self.remove_linear_momentum = remove_linear_momentum
+        self.engine = engine
 
     def _verify_snapshot(self, snapshot):
         """
@@ -265,6 +266,15 @@ class GeneralizedDirectionModifier(SnapshotModifier):
         snapshot : :class:`.Snapshot`
             input snapshot to check for validity
         """
+        try:
+            has_constraints = self.engine.has_constraints()
+        except AttributeError:
+            pass
+        else:
+            if has_constraints:
+                raise RuntimeError("Cannot use this snapshot modifier with "
+                                   "an engine that has constraints.")
+
         try:
             box_vectors = snapshot.box_vectors
         except AttributeError:
@@ -384,7 +394,7 @@ class GeneralizedDirectionModifier(SnapshotModifier):
             velocities adjusted to have the desired kinetic energy
         """
         # from here, we're doing the KE rescaling
-        # can't just use the dot product because of simtk.units
+        # can't just use the dot product because of openmm.unit
         momenta = velocities * masses[:, np.newaxis]
         dof_ke = momenta * velocities
         zero_energy = 0 * dof_ke[0][0]
